"""A docling OCR plugin for GLM-OCR."""

__version__ = "0.1.0"

from docling_glm_ocr.model import GlmOcrRemoteModel
from docling_glm_ocr.options import GlmOcrRemoteOptions

__all__ = ["GlmOcrRemoteModel", "GlmOcrRemoteOptions"]
